@@ -35,13 +35,8 @@
 
 (require 'elmo-msgdb)
 (eval-when-compile
-  (condition-case nil
-      (progn
-	(require 'starttls))
-    (error))
   (require 'elmo-cache)
-  (require 'elmo-util)
-  (defun-maybe starttls-negotiate (a)))
+  (require 'elmo-util))
 (require 'elmo-net)
 
 (eval-and-compile
@@ -1141,7 +1136,10 @@ Returns a list of cons cells like (NUMBER . VALUE)"
 	    (if (not (string= postfix ""))
 		(save-excursion
 		  (replace-regexp "^\\(211 [0-9]+ [0-9]+ [0-9]+ [^ \n]+\\).*$"
-				  (concat "\\1" postfix)))))
+				  (concat "\\1"
+					  (elmo-replace-in-string
+					   postfix
+					   "\\\\" "\\\\\\\\\\\\\\\\"))))))
 	  (let (len min max group)
 	    (while (not (eobp))
 	      (condition-case ()
