@@ -235,6 +235,7 @@ File content is encoded with MIME-CHARSET."
 	(setq auth (if (match-beginning 4)
 		       (intern (elmo-match-substring 4 folder 1))
 		     elmo-default-imap4-authenticate-type))
+	(setq auth (or auth 'clear))
 	(append (list 'imap4
 		      (elmo-imap4-encode-folder-string mailbox)
 		      user auth)
@@ -418,6 +419,7 @@ File content is encoded with MIME-CHARSET."
       (setq auth (if (match-beginning 3)
 		     (intern (elmo-match-substring 3 folder 1))
 		   elmo-default-pop3-authenticate-type))
+      (setq auth (or auth 'user))
       (append (list 'pop3 user auth)
 	      (cdr spec)))))
 
@@ -472,6 +474,12 @@ File content is encoded with MIME-CHARSET."
 	  (elmo-match-string 2 folder)
 	  (elmo-match-string 3 folder))))
 
+(defsubst elmo-pipe-spec-src (spec)
+  (nth 1 spec))
+
+(defsubst elmo-pipe-spec-dst (spec)
+  (nth 2 spec))
+
 (defun elmo-folder-get-spec (folder)
   "Return spec of FOLDER."
   (let ((type (elmo-folder-get-type folder)))
@@ -899,6 +907,31 @@ Otherwise treat \\ in NEWTEXT string as special:
    (goto-char (point-min))
    (read (current-buffer))))
 
+(defun elmo-list-to-string (list)
+  (let ((tlist list)
+	str)
+    (if (listp tlist)
+	(progn
+	  (setq str "(")
+	  (while (car tlist)
+	    (setq str
+		  (concat str
+			  (if (symbolp (car tlist))
+			      (symbol-name (car tlist))
+			    (car tlist))))
+	    (if (cdr tlist)
+		(setq str
+		      (concat str " ")))
+	    (setq tlist (cdr tlist)))
+	  (setq str
+		(concat str ")")))
+      (setq str 
+	    (if (symbolp tlist)
+		(symbol-name tlist)
+	      tlist)))
+    str))
+ 
+
 (defun elmo-plug-on-by-servers (alist &optional servers)
   (let ((server-list (or servers elmo-plug-on-servers)))
     (catch 'plugged
@@ -1085,8 +1118,12 @@ Otherwise treat \\ in NEWTEXT string as special:
 
 (defun elmo-folder-local-p (folder)
   "Return whether FOLDER is a local folder or not."
-  (let ((type (elmo-folder-get-type folder)))
-    (memq type '(localdir localnews archive maildir internal cache))))
+  (let ((spec (elmo-folder-get-spec folder)))
+    (case (car spec)
+      (filter (elmo-folder-local-p (nth 2 spec)))
+      (pipe (elmo-folder-local-p (elmo-pipe-spec-dst spec)))
+      (t (memq (car spec)
+	       '(localdir localnews archive maildir internal cache))))))
 
 (defun elmo-folder-writable-p (folder)
   (let ((type (elmo-folder-get-type folder)))
@@ -1205,6 +1242,14 @@ Otherwise treat \\ in NEWTEXT string as special:
 	(t
 	 (elmo-folder-direct-copy-p folder1 folder2))))
 
+(defun elmo-folder-get-store-type (folder)
+  (let ((spec (elmo-folder-get-spec folder)))
+    (case (car spec)
+      (filter (elmo-folder-get-store-type (nth 2 spec)))
+      (pipe (elmo-folder-get-store-type (elmo-pipe-spec-dst spec)))
+      (multi (elmo-folder-get-store-type (nth 1 spec)))
+      (t (car spec)))))
+
 (defconst elmo-folder-direct-copy-alist
   '((localdir  . (localdir localnews archive))
     (maildir   . (maildir  localdir localnews archive))
@@ -1213,8 +1258,8 @@ Otherwise treat \\ in NEWTEXT string as special:
     (cache     . (localdir localnews archive))))
 
 (defun elmo-folder-direct-copy-p (src-folder dst-folder)
-  (let ((src-type (car (elmo-folder-get-spec src-folder)))
-	(dst-type (car (elmo-folder-get-spec dst-folder)))
+  (let ((src-type (elmo-folder-get-store-type src-folder))
+	(dst-type (elmo-folder-get-store-type dst-folder))
 	dst-copy-type)
     (and (setq dst-copy-type
 	       (cdr (assq src-type elmo-folder-direct-copy-alist)))
