@@ -280,9 +280,11 @@ Matched address lists are append to CL."
 	(completing-read "To: " cl)
       (read-string "To: "))))
 
+(defconst wl-address-specials-regexp "[]\"(),.:;<>@[\\]")
+
 (defun wl-address-quote-specials (word)
   "Make quoted string of WORD if needed."
-  (if (assq 'specials (std11-lexical-analyze word))
+  (if (string-match wl-address-specials-regexp word)
       (prin1-to-string word)
     word))
 
