@@ -59,9 +59,6 @@
 (make-variable-buffer-local 'wl-message-buffer-cur-folder)
 (make-variable-buffer-local 'wl-message-buffer-cur-number)
 
-(require 'product)
-(product-provide (provide 'wl-message) (require 'wl-version))
-
 (defvar wl-fixed-window-configuration nil)
 
 (defun wl-message-buffer-window ()
@@ -141,7 +138,7 @@
 (defun wl-message-follow-current-entity (buffer)
   "Follow to current message."
   (wl-draft-reply (wl-message-get-original-buffer)
-		  'to-all wl-message-buffer-cur-summary-buffer)
+		  nil wl-message-buffer-cur-summary-buffer) ; reply to all
   (let ((mail-reply-buffer buffer))
     (wl-draft-yank-from-mail-reply-buffer nil)))
 
@@ -185,6 +182,14 @@
 	  (select-window (get-buffer-window summary-buf))))
     (run-hooks 'wl-message-exit-hook)))
 
+(defvar wl-message-mode-map nil)
+(if wl-message-mode-map
+    ()
+  (setq wl-message-mode-map (make-sparse-keymap))
+  (define-key wl-message-mode-map "q" 'wl-message-exit)
+  (define-key wl-message-mode-map "n" 'wl-message-exit)
+  (define-key wl-message-mode-map "p" 'wl-message-exit))
+
 (defun wl-message-decode (outbuf inbuf flag)
   (cond
    ((eq flag 'all-header)
@@ -207,9 +212,7 @@
 	  (elmo-set-buffer-multibyte nil))
 	(copy-to-buffer outbuf (point-min) (point-max))
 	(set-buffer outbuf)
-	(local-set-key "q" 'wl-message-exit)
-	(local-set-key "p" 'wl-message-exit)
-	(local-set-key "n" 'wl-message-exit)
+	(use-local-map wl-message-mode-map)
 	(elmo-set-buffer-multibyte default-enable-multibyte-characters)
 ;;;	(decode-mime-charset-region (point-min) (point-max) wl-mime-charset)
 	;; we can call decode-coding-region() directly, because multibyte flag is t.
@@ -283,7 +286,11 @@
 	  (wl-message-narrow-to-page 1)
 	  (setq ret-val nil))
       (condition-case ()
-	  (scroll-up lines)
+	  (static-if (boundp 'window-pixel-scroll-increment)
+	      ;; XEmacs 21.2.20 and later.
+	      (let (window-pixel-scroll-increment)
+		(scroll-up lines))
+	    (scroll-up lines))
 	(end-of-buffer
 	 (goto-char (point-max))))
       (setq ret-val nil))
@@ -636,6 +643,7 @@ If failed, attempt to execute button-dispatcher."
 	(set-buffer buf)
 	filename))))
 
+(require 'product)
+(product-provide (provide 'wl-message) (require 'wl-version))
+
 ;;; wl-message.el ends here
-
-
