@@ -33,8 +33,6 @@
 ;;; Code:
 ;;
 
-(require 'product)
-(product-provide (provide 'wl-util) (require 'wl-version))
 (eval-when-compile
   (require 'elmo-util))
 
@@ -43,7 +41,6 @@
 
 (eval-when-compile
   (require 'time-stamp)
-  (defalias-maybe 'read-event 'ignore)
   (defalias-maybe 'next-command-event 'ignore)
   (defalias-maybe 'event-to-character 'ignore)
   (defalias-maybe 'key-press-event-p 'ignore)
@@ -151,28 +148,6 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
       (append list (list element))
     list))
 
-(defun wl-read-event-char ()
-  "Get the next event."
-  (let ((event (read-event)))
-    ;; should be gnus-characterp, but this can't be called in XEmacs anyway
-    (cons (and (numberp event) event) event)))
-
-(defun wl-xmas-read-event-char ()
-  "Get the next event."
-  (let ((event (next-command-event)))
-    (sit-for 0)
-    ;; We junk all non-key events.  Is this naughty?
-    (while (not (or (key-press-event-p event)
-		    (button-press-event-p event)))
-      (dispatch-event event)
-      (setq event (next-command-event)))
-    (cons (and (key-press-event-p event)
-	       (event-to-character event))
-	  event)))
-
-(if wl-on-xemacs
-    (fset 'wl-read-event-char 'wl-xmas-read-event-char))
-
 (defmacro wl-push (v l)
   "Insert V at the head of the list stored in L."
   (list 'setq l (list 'cons v l)))
@@ -901,4 +876,7 @@ This function is imported from Emacs 20.7."
 			notify-minibuf)
 	(setq wl-biff-check-folders-running nil)))))
 
+(require 'product)
+(product-provide (provide 'wl-util) (require 'wl-version))
+
 ;;; wl-util.el ends here
