@@ -32,6 +32,8 @@
 ;;
 
 (require 'elmo2)
+(require 'wl-version)			; reduce recursive-load-depth
+
 ;; from x-face.el
 (unless (and (fboundp 'defgroup)
              (fboundp 'defcustom))
@@ -43,7 +45,6 @@
 
 (require 'wl-vars)
 (require 'wl-util)
-(require 'wl-version)
 
 (cond (wl-on-xemacs
        (require 'wl-xmas))
@@ -54,7 +55,7 @@
       (t
        (require 'wl-mule)))
 
-(provide 'wl) ; circular dependency
+(provide 'wl)				; circular dependency
 (require 'wl-folder)
 (require 'wl-summary)
 (require 'wl-thread)
@@ -714,6 +715,10 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
 					"." wl-local-domain)
 			      (system-name))))
       (error "Please remove `@' from `wl-message-id-domain'"))
+  (if (string= wl-local-domain "localdomain")
+      (error "Please set `wl-local-domain'"))
+  (if (string= wl-message-id-domain "localhost.localdomain")
+      (error "Please set `wl-message-id-domain'"))
   ;; folders
   (when (not no-check-folder)
     (if (not (eq (elmo-folder-get-type wl-draft-folder) 'localdir))
